# Copyright (C) 2015 Brent Baude <bbaude@redhat.com>
# Copyright (C) 2019 Dominique Blaze <contact@d0m.tech>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301 USA

import subprocess
import platform
import os
import collections
import re
from typing import List


class OscapError(Exception):
    ''' oscap Error'''
    pass


OscapResult = collections.namedtuple("CmdResult", ("returncode", "stdout", "stderr"))
DistroInfo = collections.namedtuple("DistroInfo", ("name", "version", "oval_url"))


def oscap_chroot(chroot_path, oscap_binary, oscap_args, target_name, local_env=[]):
    '''
    Wrapper running oscap_chroot on an OscapDockerScan OscapAtomicScan object
    '''
    os.environ["OSCAP_PROBE_ROOT"] = os.path.join(chroot_path)
    os.environ["OSCAP_EVALUATION_TARGET"] = target_name
    os.environ["OSCAP_CONTAINER_VARS"] = '\n'.join(local_env)
    cmd = [oscap_binary] + [x for x in oscap_args]
    return run_cmd(cmd)


def get_distro_info(mount_point: str) -> DistroInfo:
    cmd = ['cat', os.path.join(mount_point, 'etc', 'system-release-cpe')]
    result = run_cmd(cmd)
    if result.stderr:
        raise Exception(result.stderr)
    res = re.search(r'cpe:/([oah]):([^:]+):([^:]+):([^:]+)(?::([^:]*)(?::([^:]*)(?::([^:]*))?)?)?',
                    result.stdout)
    if res:
        os_name, os_version = res.group(3), res.group(4)
        if os_name and os_version:
            if os_name == 'msvsphere':
                edition = res.group(6)
                if edition == 'baseos':
                    return DistroInfo(os_name,
                                      os_version,
                                      f'https://repo1.msvsphere-os.ru/msvsphere/'
                                      f'{os_version}/security/oval/msvsphere-9.en.oval.xml')
                elif edition == 'certified':
                    return DistroInfo(os_name,
                                      os_version,
                                      f'https://repo1.msvsphere-os.ru/certified/msvsphere/'
                                      f'{os_version}/security/oval/msvsphere-9.en.oval.xml')
                elif not edition:
                    raise Exception(f'There is no edition in CPE_NAME')
                else:
                    raise Exception(f'Invalid CPE edition: {edition}')
            elif os_name == 'enterprise_linux':
                return DistroInfo(os_name,
                                  os_version,
                                  f'https://security.access.redhat.com/data/oval/v2/RHEL'
                                  f'{os_version}/rhel-{os_version}.oval.xml.bz2')
            raise Exception(f"Distribution {os_name} {os_version} is not supported")
    raise Exception(f'Invalid CPE_NAME: {result.stdout}')


def run_cmd(cmd: List[str]) -> OscapResult:
    cmd_process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    cmd_stdout, cmd_stderr = cmd_process.communicate()
    return OscapResult(cmd_process.returncode,
                       cmd_stdout.decode("utf-8").rstrip(),
                       cmd_stderr.decode("utf-8"))
